/*
 * Minecraft Forge
 * Copyright (c) 2016.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version 2.1
 * of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.minecraftforge.items;

import net.minecraft.block.BlockDropper;
import net.minecraft.block.BlockHopper;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.IHopper;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityDispenser;
import net.minecraft.tileentity.TileEntityHopper;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;

public class VanillaInventoryCodeHooks
{

    public static boolean extractHook(IHopper dest)
    {
        TileEntity tileEntity = dest.func_145831_w().func_175625_s(new BlockPos(dest.func_96107_aA(), dest.func_96109_aB() + 1, dest.func_96108_aC()));

        if (tileEntity == null || !tileEntity.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, EnumFacing.DOWN))
            return false;

        IItemHandler handler = tileEntity.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, EnumFacing.DOWN);

        for (int i = 0; i < handler.getSlots(); i++)
        {
            ItemStack extractItem = handler.extractItem(i, 1, true);
            if (extractItem != null)
            {
                for (int j = 0; j < dest.func_70302_i_(); j++)
                {
                    ItemStack destStack = dest.func_70301_a(j);
                    if (destStack == null || destStack.field_77994_a < destStack.func_77976_d() && ItemHandlerHelper.canItemStacksStack(extractItem, destStack))
                    {
                        extractItem = handler.extractItem(i, 1, false);
                        if (destStack == null)
                            dest.func_70299_a(j, extractItem);
                        else
                        {
                            destStack.field_77994_a++;
                            dest.func_70299_a(j, destStack);
                        }
                        dest.func_70296_d();
                        return true;
                    }
                }
            }
        }

        return true;
    }

    public static boolean dropperInsertHook(World world, BlockPos pos, TileEntityDispenser dropper, int slot, ItemStack stack)
    {
        EnumFacing enumfacing = world.func_180495_p(pos).func_177229_b(BlockDropper.field_176441_a);
        BlockPos offsetPos = pos.func_177972_a(enumfacing);
        TileEntity tileEntity = world.func_175625_s(offsetPos);
        if (tileEntity == null)
            return true;
        if (!tileEntity.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, enumfacing.func_176734_d()))
            return true;

        IItemHandler capability = tileEntity.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, enumfacing.func_176734_d());

        ItemStack result = ItemHandlerHelper.insertItem(capability, ItemHandlerHelper.copyStackWithSize(stack, 1), false);

        if (result == null)
        {
            result = stack.func_77946_l();

            if (--result.field_77994_a == 0)
            {
                result = null;
            }
        }
        else
        {
            result = stack.func_77946_l();
        }
        dropper.func_70299_a(slot, result);
        dropper.func_70296_d();
        return false;
    }

    public static boolean insertHook(TileEntityHopper hopper)
    {
        return insertHook(hopper, BlockHopper.func_176428_b(hopper.func_145832_p()));
    }

    public static boolean insertHook(IHopper hopper, EnumFacing facing)
    {
        TileEntity tileEntity = hopper.func_145831_w().func_175625_s(
                new BlockPos(hopper.func_96107_aA(), hopper.func_96109_aB(), hopper.func_96108_aC()).func_177972_a(facing));

        if (tileEntity == null)
            return false;
        if (!tileEntity.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, facing.func_176734_d()))
            return false;

        IItemHandler handler = tileEntity.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, facing.func_176734_d());

        for (int i = 0; i < hopper.func_70302_i_(); i++)
        {
            ItemStack stackInSlot = hopper.func_70301_a(i);
            if (stackInSlot != null)
            {
                ItemStack insert = stackInSlot.func_77946_l();
                insert.field_77994_a = 1;
                ItemStack newStack = ItemHandlerHelper.insertItem(handler, insert, true);
                if (newStack == null || newStack.field_77994_a == 0)
                {
                    ItemHandlerHelper.insertItem(handler, hopper.func_70298_a(i, 1), false);
                    hopper.func_70296_d();
                    return true;
                }
            }
        }

        return true;
    }
}
